package org.xbib.event.generic;

import java.io.IOException;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

import org.xbib.event.bus.EventBus;
import org.xbib.event.bus.Subscriber;
import org.xbib.event.bus.SubscriberRegistry;
import org.xbib.event.common.EventManager;
import org.xbib.event.common.EventManagerService;

public class GenericEventManagerService implements EventManagerService {

    private EventBus eventBus;

    public GenericEventManagerService() {
    }

    @Override
    public GenericEventManagerService init(EventManager eventManager) {
        this.eventBus = eventManager.getEventBus();
        return this;
    }

    @Override
    public void shutdown() throws IOException {

    }

    public void post(Object event) {
        eventBus.post(event);
    }

    public void post(GenericEventImpl event,
                     CompletableFuture<GenericEvent> future) {
        SubscriberRegistry subscriberRegistry = eventBus.getSubscribers();
        Set<Subscriber> set = subscriberRegistry.getSubscribersForTesting(event.getClass());
        event.setListener(new WrappedListener(event.getListener(), set.size(), future));
        post(event);
    }

    static class WrappedListener implements Listener {

        private final Listener listener;

        private int size;

        private final CompletableFuture<GenericEvent> future;

        public WrappedListener(Listener listener, int size, CompletableFuture<GenericEvent> future) {
            this.listener = listener;
            this.size = size;
            this.future = future;
        }

        @Override
        public void listen(GenericEvent event) {
            if (listener != null) {
                listener.listen(event);
            }
            if (--size == 0) {
                future.complete(event);
            }
        }
    }
}
